var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// private properties
const componentsByTagName = new Map();
const tagDefsByTagName = new Map();
// public function
export const register = (tagDef, mode = 'open') => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    if (!customElements) {
        throw new Error('Custom elements is not supported by your browser!');
    }
    if (tagDef.templateUrl) {
        const response = yield fetch(tagDef.templateUrl);
        const div = document.createElement('div');
        div.innerHTML = yield response.text();
        tagDef.template = div.firstChild;
        if (!tagDef.template.content) {
            throw (`Failed to create template from\n${div.innerHTML}`);
        }
    }
    tagDefsByTagName.set(tagDef.tagName, tagDef);
    // check if we are rendering server side
    const shadowRoot = (_a = document.querySelector(tagDef.tagName)) === null || _a === void 0 ? void 0 : _a.shadowRoot;
    const isSsr = !!((_b = document.querySelector(tagDef.tagName)) === null || _b === void 0 ? void 0 : _b.shadowRoot);
    if (!isSsr && tagDef.loadData) {
        // if we haven't rendered this component and it needs data load the data
        tagDef.hydrated = yield tagDef.loadData();
    }
    const wrapper = class Wrapper extends HTMLElement {
        constructor() {
            // Always call super first in constructor
            super();
            // constructed has to be called before the component is inserted
            if (tagDef.constructed) {
                tagDef.constructed(isSsr);
            }
            // Create a shadow root
            const shadow = shadowRoot || this.attachShadow({ mode: mode });
            // create our component
            const component = tagDef.tagFunction();
            // if a tag def template is supplied import the node, else use the shadow root. This allows for inline elements.
            const clone = tagDef.template ? document.importNode(tagDef.template.content, true)
                : document.importNode(this.querySelector('template').content, true);
            component.element = isSsr ? shadowRoot.querySelector('[data-component-root="root"]')
                : clone.querySelector('[data-component-root="root"]');
            // @ts-ignore
            const styles = [...clone.childNodes].find((child) => {
                if (child.nodeType === Node.TEXT_NODE) {
                    return false;
                }
                return child.tagName.toLowerCase() === 'style';
            });
            if (!componentsByTagName.get(tagDef.tagName)) {
                componentsByTagName.set(tagDef.tagName, []);
            }
            // store the component instances by tag name so an observer can assign mediators
            // for a surrounding application
            componentsByTagName.get(tagDef.tagName).push(component);
            const renderedComponent = component.render([], isSsr);
            // TODO add lifecycle hooks
            // Attach the created elements to the shadow dom if it hasn't been rendered server side
            if (!isSsr && styles) {
                const style = document.createElement('style');
                style.textContent = styles.textContent;
                shadow.appendChild(style);
            }
            shadow.appendChild(renderedComponent);
            this.component = component;
        }
        connectedCallback() {
            console.log(`tag ${tagDef.tagName} inserted`);
            tagDef.inserted(this.component);
        }
        disconnectedCallback() {
            console.log(`tag ${tagDef.tagName} removed`);
            // remove the component instance
            const index = componentsByTagName.get(tagDef.tagName).findIndex((instance) => instance === this.component);
            componentsByTagName.get(tagDef.tagName).splice(index, 1);
            // destroy the component, this removes event listeners etc
            this.component.destroy(true);
            tagDef.removed(this.component);
            tagDef.template = null;
        }
    };
    customElements.define(tagDef.tagName, wrapper);
});
export const getTagDef = (tagName) => {
    return tagDefsByTagName.get(tagName);
};
//# sourceMappingURL=ComponentRegistry.js.map