var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import composeP from 'ramda/es/composeP';
import compose from 'ramda/es/compose';
const subscriberMap = new Map();
// used to intercept events, cause side effects, and forward to subscribers
// SHOULD BE INTERNAL ONLY! Do not expose this to the public.
const interceptors = [];
// factories
const trace = function (value) {
    // eslint-disable-next-line no-console
    console.log(`event bus invoked with event type: ${value.type} and payload: ${value.payload}`);
    return value;
};
const registerFactory = function (subscriberMap) {
    return (event, onNext, interceptor) => {
        if (!subscriberMap.get(event.type)) {
            subscriberMap.set(event.type, []);
        }
        if (interceptor) {
            interceptors.push(interceptor);
        }
        subscriberMap.get(event.type).push(onNext);
        // operation is idempotent
        return {
            closed: false,
            unsubscribe() {
                if (interceptor) {
                    const interceptorIndex = interceptors.indexOf(interceptor);
                    interceptors.splice(interceptorIndex, 1);
                }
                const index = subscriberMap.get(event.type).indexOf(onNext);
                this.closed = true;
                // splice mutates the array in
                subscriberMap.get(event.type).splice(index, 1);
            },
        };
    };
};
const emitFactory = function (subscriberMap) {
    return function (event) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // if interceptors have not canceled the event fire it
            if (!(event === null || event === void 0 ? void 0 : event.cancel)) {
                (_a = subscriberMap
                    .get(event.type)) === null || _a === void 0 ? void 0 : _a.forEach((callback) => {
                    callback(event);
                });
            }
            return event;
        });
    };
};
const interceptorFactory = function (interceptors) {
    interceptors.push(trace);
    return function (event) {
        return __awaiter(this, void 0, void 0, function* () {
            // have interceptors receive and transform the event
            interceptors.forEach(interceptor => {
                interceptor(event);
            });
            // if interceptors have not canceled the event fire it
            return event;
        });
    };
};
const promiseFactory = function (event) {
    return __awaiter(this, void 0, void 0, function* () {
        let result;
        if (event instanceof Promise) {
            result = yield event;
        }
        else {
            result = event;
        }
        return result;
    });
};
// exports
export const emit = composeP(emitFactory(subscriberMap), interceptorFactory(interceptors), promiseFactory);
// in the future I think we could have some more interceptors besides trace
// please note that registerFactory must be last in composition due to the number of args
export const listen = compose(registerFactory)(subscriberMap);
//# sourceMappingURL=EventBus.js.map