var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// typing the map as Map<string, State | State[]> causes the compiler to complain insistently
function doTransaction(states, timeout = 1000, values = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            const transitions = new Map(states);
            try {
                const state = transitions.get('current');
                if (state && !state.hasOwnProperty('executed')) {
                    try {
                        // TODO consider returning a new state as part of execute.
                        //  I'd like to make sure we don't cause side effects either to
                        //  the original states, or the states array
                        const result = yield state.execute(values);
                        state.executed = 1;
                        state.result = result;
                        transitions.get('stack').push(state);
                        if (state.next) {
                            transitions.set('current', state.next(state, transitions.get('states')));
                            yield doTransaction(transitions, timeout, result);
                        }
                    }
                    catch (e) {
                        if (state.error) {
                            const result = Object.assign({ error: undefined }, values);
                            result.error = e;
                            state.executed = -1;
                            state.result = result;
                            transitions.set('current', state.error(state, transitions.get('states')));
                            yield doTransaction(transitions, timeout, result);
                        }
                    }
                }
                resolve({ transitions });
            }
            catch (e) {
                reject(e);
            }
        }));
    });
}
export function stateMachine(states, timeout = 1000) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO make cancelable
        const transitions = new Map();
        transitions.set('stack', []);
        transitions.set('states', states);
        transitions.set('current', states[0]);
        return yield doTransaction(transitions, timeout);
    });
}
//# sourceMappingURL=SimpleFsm.js.map