// utils
export const random = () => {
    return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
};
// enums
export var Events;
(function (Events) {
    Events["CLICK"] = "lotusComponentClick";
    Events["ITEM_SELECTED"] = "lotusItemViewItemSelected";
    Events["ITEM_DESELECTED"] = "lotusItemViewItemDeselected";
    Events["REMOVE_ITEM"] = "lotusItemViewRemoveItem";
})(Events || (Events = {}));
// const functions
export const addProperty = (instance, label, getter, setter, enumerable = true) => {
    Object.defineProperty(instance, label, {
        get: getter,
        set: setter,
        enumerable: enumerable
    });
    return instance;
};
export const mixin = (target, sub) => {
    // IMPORTANT: mixin is designed to function like Object.assign, just respects accessors
    // grab enumerable properties of the target object
    const keys = Object.keys(sub);
    keys.forEach((prop) => {
        const methodDef = Object.getOwnPropertyDescriptor(sub, prop);
        // check if this property uses accessor methods (Object.assign can't do this!)
        if (methodDef && (methodDef.get || methodDef.set)) {
            // yep
            addProperty(target, prop, Object.getOwnPropertyDescriptor(sub, prop).get, Object.getOwnPropertyDescriptor(sub, prop).set);
        }
        else {
            // nope
            target[prop] = sub[prop];
        }
    });
    return target;
};
export const getTemplate = () => {
    return {
        // placeholders for mixins and interfaces, required for the compiler
        handlersByEventName: new Map(),
        addEventListener: (event, instance, handler) => null,
        canListen: (event, instance, handler) => null,
        removeEventListener: (event, instance, handler) => null,
        removeEventListeners: () => null,
        removeAllEventListeners: (instance) => null,
        dispatch: (event) => null,
        ready: false,
        id: random(),
        element: null,
        destroy: null,
        init: null,
        inserted: null,
        removed: null,
        attributeChanged: null,
        onSkinPartAdded: (part) => null,
        attributeMap: new Map(),
        skinPartMap: new Map(),
        addAttributes: null,
        addSkinParts: null,
        render: null,
    };
};
export const getComponentEvent = (type, payload) => {
    return {
        type,
        payload,
        clone: (event) => {
            return Object.assign({}, event);
        }
    };
};
export const createComponent = () => {
    let clone = getTemplate();
    const _id = random();
    clone.addEventListener = (event, instance, handler) => {
        if (!clone.handlersByEventName.get(event)) {
            clone.handlersByEventName.set(event, []);
        }
        clone.handlersByEventName.get(event).push({ handler, instance });
    };
    clone.canListen = (eventType, instance, handler) => {
        let canListen = false;
        if (clone.handlersByEventName.get(eventType)) {
            for (let handlerIndex = 0; handlerIndex < clone.handlersByEventName.get(eventType).length; handlerIndex++) {
                const handlerFunctionName = clone.handlersByEventName.get(eventType)[handlerIndex].handler;
                const objectInstance = clone.handlersByEventName.get(eventType)[handlerIndex].instance;
                if (handlerFunctionName == handler && objectInstance == instance) {
                    canListen = true;
                    break;
                }
            }
        }
        return canListen;
    };
    clone.removeEventListener = (event, instance, handler) => {
        if (!clone.handlersByEventName.get(event)) {
            return;
        }
        for (let handlerIndex = 0; handlerIndex < clone.handlersByEventName.get(event).length; handlerIndex++) {
            if (clone.handlersByEventName.get(event)[handlerIndex].instance == instance && clone.handlersByEventName.get(event)[handlerIndex].handler == handler) {
                const itemToRemove = clone.handlersByEventName.get(event)[handlerIndex];
                switch (handlerIndex) {
                    case 0:
                        clone.handlersByEventName.get(event).shift();
                        break;
                    case clone.handlersByEventName.get(event).length - 1:
                        clone.handlersByEventName.get(event).pop();
                        break;
                    default:
                        const head = clone.handlersByEventName.get(event).slice(0, handlerIndex);
                        const tail = clone.handlersByEventName.get(event).slice(handlerIndex + 1);
                        clone.handlersByEventName.set(event, head.concat(tail));
                        break;
                }
                // there can be only one item matching event, instance, handler so we return here
                return itemToRemove;
            }
        }
    };
    clone.removeAllEventListeners = (instance) => {
        for (const event in clone.handlersByEventName) {
            clone.handlersByEventName.get(event).forEach((listener) => {
                if (listener.instance == instance) {
                    clone.removeEventListener(event, instance, listener.handler);
                }
            });
        }
    };
    clone.dispatch = (event) => {
        if (clone.handlersByEventName.get(event.type) === null || clone.handlersByEventName.get(event.type) === undefined) {
            return;
        }
        // We need to make a copy of event handles before dispatching.
        // If the handler removes itself from the event queue during dispatching, it triggers removeEventListener, which
        // changes the array and this messes up the entire dispatch process (some handlers are never called).
        const dispatchToList = clone.handlersByEventName.get(event.type).slice();
        const len = dispatchToList.length;
        for (let handlerIndex = 0; handlerIndex < len; ++handlerIndex) {
            const handlerFunctionName = dispatchToList[handlerIndex].handler;
            const instance = dispatchToList[handlerIndex].instance;
            // @ts-ignore
            instance[handlerFunctionName](event);
        }
    };
    clone.addSkinParts = () => {
        if (clone.element.getAttribute('data-skin-part') !== null
            && clone.element.getAttribute('data-skin-part') !== undefined) {
            clone.skinPartMap.set(clone.element.getAttribute('data-skin-part'), clone.element);
            clone.onSkinPartAdded(clone.element.getAttribute('data-skin-part'));
        }
        const skinPartsNodeList = clone.element.querySelectorAll('[data-skin-part]');
        for (let i = 0; i < skinPartsNodeList.length; i++) {
            clone.skinPartMap.set(skinPartsNodeList[i].getAttribute('data-skin-part'), skinPartsNodeList[i]);
            clone.onSkinPartAdded(skinPartsNodeList[i].getAttribute('data-skin-part'));
        }
    };
    clone.addAttributes = () => {
        // only get enumerable properties
        const properties = Object.keys(clone);
        for (let i = 0; i < clone.element.attributes.length; i++) {
            const attribute = clone.element.attributes[i];
            if (attribute.name.indexOf('attribute') >= 0) {
                const index = attribute.name.indexOf('attribute') + 10;
                const newProp = attribute.name.substring(index); // remove prefix
                // convert dashes to camel case
                const camelCased = newProp.replace(/-([a-z])/g, function (g) {
                    return g[1].toUpperCase();
                });
                if (properties.indexOf(camelCased) >= 0) {
                    clone.attributeMap.set(clone[camelCased], attribute.value);
                }
            }
        }
    };
    clone.destroy = (removed = false) => {
        if (!clone.ready) {
            return;
        }
        clone.removeEventListeners();
        clone.attributeMap.clear();
        clone.skinPartMap.clear();
        // only make clone eligible for GC if removed from DOM
        if (removed) {
            clone = null;
        }
    };
    clone.render = (list, isSsr = false) => {
        // be sure to call removeEventListeners so the old element is garbage collected
        // if you don't remove event listeners the GC will not collect the element
        if (!isSsr) {
            clone.destroy();
            clone.element = clone.element.cloneNode(true);
        }
        clone.addAttributes();
        clone.addSkinParts();
        clone.ready = true;
        return clone.element;
    };
    addProperty(clone, 'id', function () {
        return _id;
    });
    // TODO create functional event dispatch and replace new EventDispatcher()
    return clone;
};
//# sourceMappingURL=AbstractComponent.js.map