import { createComponent as createAbstractComponent, mixin } from './AbstractComponent';
import { createComponent as createCollectionComponent, createItemView } from './AbstractCollectionComponent';
import compose from 'ramda/es/compose';
export const getScaleToFit = (objSize, sizeToFit) => {
    return Math.min(sizeToFit.width / objSize.width, sizeToFit.height / objSize.height);
};
// export public functions
export const createImageView = (component) => {
    // TODO figure out which of the functions below (if any) should be private
    const clone = mixin(component, {
        width: NaN,
        height: NaN,
        allowDrag: true,
        onThumbClick: false,
        onDragStart: false,
        onImageLoad: false,
        setThumbnailSrc: false,
    });
    const render = clone.render;
    const destroy = clone.destroy;
    const onSkinPartAdded = clone.onSkinPartAdded;
    const resetState = clone.resetState;
    const sizeImage = () => {
        if (!clone.skinPartMap.get('thumbnail')) {
            return;
        }
        const imageSize = {
            width: parseInt(clone.skinPartMap.get('thumbnail').getAttribute('width')),
            height: parseInt(clone.skinPartMap.get('thumbnail').getAttribute('height')),
        };
        const containerSize = {
            width: parseInt(window.getComputedStyle(clone.element).width),
            height: parseInt(window.getComputedStyle(clone.element).height),
        };
        const scale = getScaleToFit(imageSize, containerSize);
        const width = imageSize.width * scale;
        const height = imageSize.height * scale;
        // console.log("width/height "+width+"/"+height)
        clone.skinPartMap.get('thumbnail').setAttribute('width', `${width}px`);
        clone.skinPartMap.get('thumbnail').setAttribute('height', `${height}px`);
        clone.skinPartMap.get('thumbnail').style.maxWidth = `${containerSize.width}px`;
        clone.skinPartMap.get('thumbnail').style.maxHeight = `${containerSize.height}px`;
    };
    clone.onThumbClick = () => {
        clone.resetState(clone.element.classList.contains('selected'));
    };
    clone.onDragStart = () => {
        // TODO: attach data attribute
    };
    clone.resetState = (state) => {
        resetState(state);
        console.log(`you clicked: ${clone.skinPartMap.get('caption').innerHTML}`);
        // TODO: handle overrides
    };
    clone.onImageLoad = () => {
        clone.skinPartMap.get('thumbnail').onload = null;
        clone.skinPartMap.get('thumbnail').style.visibility = 'visible';
        sizeImage();
    };
    clone.setThumbnailSrc = (src) => {
        clone.skinPartMap.get('thumbnail').onload = clone.onImageLoad;
        clone.skinPartMap.get('thumbnail').style.visibility = 'hidden';
        clone.skinPartMap.get('thumbnail').src = src;
    };
    clone.removeEventListeners = () => {
        clone.skinPartMap.get('thumbnail').removeEventListener('click', clone.onThumbClick);
        clone.skinPartMap.get('thumbnail').removeEventListener('dragstart', clone.onDragStart);
    };
    clone.onSkinPartAdded = (part) => {
        onSkinPartAdded(part);
        switch (part) {
            // required, defines the layout for child views
            case 'thumbnail':
                clone.skinPartMap.get('thumbnail').setAttribute('draggable', (clone.allowDrag) ? 'true' : 'false');
                clone.skinPartMap.get('thumbnail').addEventListener('click', clone.onThumbClick);
                clone.skinPartMap.get('thumbnail').addEventListener('dragstart', clone.onDragStart);
                break;
        }
    };
    clone.destroy = () => {
        if (!clone.ready) {
            return;
        }
        // IMPORTANT child components need to clean up before calling super.destroy!!!
        clone.removeEventListeners();
        destroy();
    };
    clone.render = (list, isSsr = false) => {
        render(list, isSsr);
        if (list && list.length === 1) {
            clone.setThumbnailSrc(list[0].src);
            if (clone.skinPartMap.get('caption')) {
                clone.skinPartMap.get('caption').innerHTML = list[0].caption;
            }
        }
        else if (isSsr) {
            // if pre-rendered on the server, pass the url that was generated
            clone.setThumbnailSrc(clone.skinPartMap.get('thumbnail').src);
        }
        return clone.element;
    };
    return clone;
};
export const createComponent = (component) => {
    return mixin(component, {
        title: ''
    });
};
// create hook using compose
// hooks provide prebuilt functions that are useful
export const useImageView = compose(createImageView, createItemView, createAbstractComponent);
export const useImageGallery = compose(createComponent, createCollectionComponent, createAbstractComponent);
//# sourceMappingURL=ImageGallery.js.map